extends Reference
class_name ResourceAsyncLoader

# main thread only
var _path : String
var _resource : Resource = null
var _progress : float = 0.0
var _thread : Thread = Thread.new()
# shared
var _mutex : Mutex = Mutex.new()
var _shared_result : int = OK
var _shared_progress : float = 0.0


func _init(path : String) -> void:
	assert(path != null) # and ResourceLoader.exits(path))
	self._path = path
	

func get_progress() -> float:
	return self._progress


func get_resource() -> Resource:
	return self._resource


func poll() -> int:
	var result : int

	if self._resource != null:
		result = ERR_FILE_EOF
	elif not self._thread.is_active():
		#print("START THREAD")
		result = self._thread.start(self, "_thread_run", ResourceLoader.load_interactive(self._path))
	else:
		self._lock("poll")
		result = _shared_result
		self._progress = self._shared_progress
		self._unlock("poll")

	#print("Progress: %s" % [self._progress])

	if result != OK and self._thread.is_active():
		self._progress = 1.0
		self._resource = self._thread.wait_to_finish()
	
	return result


func _thread_run(loader : ResourceInteractiveLoader) -> Resource:
	#print("THREAD STARTED")
	var run : bool = true
	while run:
		var result : int = loader.poll()
		var progress : float = (1.0 + loader.get_stage()) / float(loader.get_stage_count())
		self._lock("_thread_run")
		self._shared_result = result
		self._shared_progress = progress
		#print("Stage: %s / %s" % [loader.get_stage(), loader.get_stage_count()])
		self._unlock("_thread_run")
		if result != OK:
			#print("THREAD is FINISHED!!!")
			return loader.get_resource()
	return null
	

func _lock(_caller : String) -> void:
	#print("LOCK: " + _caller)
	self._mutex.lock()


func _unlock(_caller : String) -> void:
	self._mutex.unlock()
	#print("UNLOCK: " + _caller)

